module ConfigFile where

import Char
import System
import Control.Concurrent
import System.IO.Unsafe

type ConfigFile = [(String, String)]


getConfiguration :: String -> IO ConfigFile
getConfiguration s = do
    e <- getEnv "HOME"
    c <- readFile (e ++ "/" ++ s)
    return $ parseConfigFile c
    
dumpConfigFile :: ConfigFile -> String
dumpConfigFile cf = unlines $ map dc cf where
    dc (n,v) = n ++ " " ++ v

mergeConfigFile :: ConfigFile -> ConfigFile -> ConfigFile
mergeConfigFile c1 c2 = c1 ++ c2

isPropName c = isAlphaNum c || c `elem` "-_"

parseConfigFile :: String -> ConfigFile
parseConfigFile s = concatMap bl (lines (uncomment s)) where
    uncomment ('-':'-':xs) = uncomment (dropWhile (/= '\n') xs)
    uncomment ('/':'*':xs) = uncomment (dropEnd xs)  
    uncomment (x:xs) = x:uncomment xs
    uncomment [] = []
    dropEnd ('*':'/':xs) = uncomment xs
    dropEnd (_:xs) = dropEnd xs
    dropEnd [] = []
    bl s = let (n,r) = span isPropName (dropWhile isSpace s) in
	if null n then [] else [(n,dropWhile isSpace r)]


{-# NOINLINE config_file_var #-}
config_file_var :: MVar ConfigFile
config_file_var = unsafePerformIO $ newMVar []


loadStdConfiguration :: String -> IO ()
loadStdConfiguration f = do
    c <- getConfiguration f
    swapMVar config_file_var c
    return ()

loadExactConfiguration :: String -> IO ()
loadExactConfiguration f = do
    s <- readFile f 
    let c = parseConfigFile s
    swapMVar config_file_var c
    return ()


configLookupBool k = do
    vs <- readMVar config_file_var
    case lookup k vs of 
	Just s | cond -> return True where
	    cond = s' == "true" || s' == "yes" || s' == "on" || s' == "y" || s' == "t"
	    s' = map toLower s
	_ -> return False
    

configLookup k = do
    vs <- readMVar config_file_var
    return $ lookup k vs


configLookupList k = do
    vs <- readMVar config_file_var
    return $ [ y| (x,y) <- vs, x == k]

